@@ -16,6 +16,12 @@ class SolanaMintAddresses:
     # Make a pull request and add more! That would be cool.
 
 
+class SolanaCandyMachineContractVersion(Enum):
+    V1 = "v1"
+    V2 = "v2"
+    MAGIC_EDEN = "magic-eden-v1"
+
+
 class SolanaNetwork(Enum):
     DEVNET = "devnet"
     MAINNET_BETA = "mainnet-beta"
@@ -31,6 +37,11 @@ class SolanaNFTUploadMethod(Enum):
     LINK = "LINK"
 
 
+class SearchMethod(Enum):
+    BEGINS_WITH = "begins_with"
+    EXACT_MATCH = "exact_match"
+
+
 class TheBlockchainAPIResource:
 
     __url = "https://api.theblockchainapi.com/v1/"
@@ -196,23 +207,31 @@ class TheBlockchainAPIResource:
     def get_wallet_token_holdings(
         self,
         public_key: str,
+        include_nfts: bool = False,
+        include_zero_balance_holdings: bool = False,
         network: SolanaNetwork = SolanaNetwork.DEVNET
     ) -> list:
         """
         More info:
         https://docs.theblockchainapi.com/#operation/solanaGetTokensBelongingToWallet
         :param public_key:
+        :param include_nfts:
+        :param include_zero_balance_holdings:
         :param network:
         :return:
         """
         response = self._request(
-            payload=dict(),
+            payload={
+                'include_nfts': include_nfts,
+                'include_zero_balance_holdings': include_zero_balance_holdings
+            },
             endpoint=f"solana/wallet/{network.value}/{public_key}/tokens",
             request_method=self.__RequestMethod.GET
         )
         if 'error_message' in response:
             raise Exception(response['error_message'])
-        return response['nfts_owned']
+
+        return response
 
     def get_nfts_belonging_to_address(
         self,
@@ -431,6 +450,47 @@ class TheBlockchainAPIResource:
             raise Exception(response['error_message'])
         return response
 
+    def search_nfts(
+        self,
+        update_authority: Optional[str] = None,
+        update_authority_search_method: SearchMethod = SearchMethod.EXACT_MATCH,
+        mint_address: Optional[str] = None,
+        mint_address_search_method: SearchMethod = SearchMethod.EXACT_MATCH,
+        nft_name: Optional[str] = None,
+        nft_name_search_method: SearchMethod = SearchMethod.EXACT_MATCH,
+        nft_uri: Optional[str] = None,
+        nft_uri_search_method: SearchMethod = SearchMethod.EXACT_MATCH,
+        symbol: Optional[str] = None,
+        symbol_search_method: SearchMethod = SearchMethod.EXACT_MATCH,
+        network: SolanaNetwork = SolanaNetwork.DEVNET
+    ):
+        payload = {
+            'network': network.value
+        }
+        if update_authority is not None:
+            payload['update_authority'] = update_authority
+            payload['update_authority_search_method'] = update_authority_search_method.value
+        if mint_address is not None:
+            payload['mint_address'] = mint_address
+            payload['mint_address_search_method'] = mint_address_search_method.value
+        if nft_uri is not None:
+            payload['uri'] = nft_uri
+            payload['uri_search_method'] = nft_uri_search_method.value
+        if symbol is not None:
+            payload['symbol'] = symbol
+            payload['symbol_search_method'] = symbol_search_method.value
+        if nft_name is not None:
+            payload['name'] = nft_name
+            payload['name_search_method'] = nft_name_search_method.value
+        response = self._request(
+            payload=payload,
+            endpoint="solana/nft/candy_machine/search",
+            request_method=self.__RequestMethod.POST
+        )
+        if 'error_message' in response:
+            raise Exception(response['error_message'])
+        return response
+
     def get_nft_metadata(
         self,
         mint_address: str,
@@ -491,25 +551,38 @@ class TheBlockchainAPIResource:
             raise Exception(response['error_message'])
         return response['transaction_signature']
 
-    def get_candy_machine_info(
+    def get_candy_machine_metadata(
         self,
-        candy_machine_id: str,
+        candy_machine_id: Optional[str] = None,
+        config_address: Optional[str] = None,
+        uuid: Optional[str] = None,
+        candy_machine_contract_version: SolanaCandyMachineContractVersion = SolanaCandyMachineContractVersion.V1,
         network: SolanaNetwork = SolanaNetwork.DEVNET
     ):
         """
         More Info:
         https://docs.theblockchainapi.com/#operation/solanaGetCandyMachineDetails
-        :param candy_machine_id:
-        :param network:
+        :param candy_machine_id: The candy_machine_id. Same as config_address in v2.
+        :param config_address: The config_address. Same as candy_machine_id in v2.
+        :param uuid: The first six characters of config_address. Sometimes, you can only find the uuid.
+        :param candy_machine_contract_version: The version of the candy machine you're querying. If you're querying v1
+        but the candy machine is v2, then you will get a not found error, so the version is important.
+        :param network: e.g., mainnet-beta, devnet
         :return:
         """
         payload = {
             "network": network.value,
-            "candy_machine_id": candy_machine_id
+            "candy_machine_contract_version": candy_machine_contract_version.value
         }
+        if candy_machine_id is not None:
+            payload['candy_machine_id'] = candy_machine_id
+        if config_address is not None:
+            payload['config_address'] = config_address
+        if uuid is not None:
+            payload['uuid'] = uuid
         response = self._request(
             payload=payload,
-            endpoint="solana/nft/candy_machine/info",
+            endpoint="solana/nft/candy_machine/metadata",
             request_method=self.__RequestMethod.POST
         )
         if 'error_message' in response:
@@ -522,6 +595,7 @@ class TheBlockchainAPIResource:
         secret_recovery_phrase: str,
         derivation_path: Optional[str] = None,
         passphrase: str = str(),
+        candy_machine_contract_version: SolanaCandyMachineContractVersion = SolanaCandyMachineContractVersion.V1,
         network: SolanaNetwork = SolanaNetwork.DEVNET
     ):
         """
@@ -534,6 +608,7 @@ class TheBlockchainAPIResource:
         :param secret_recovery_phrase:
         :param derivation_path:
         :param passphrase:
+        :param candy_machine_contract_version:
         :param network:
         :return: A task_id. Use the `get_task` function to retrieve the result once this task has completed processing.
         You can poll the `get_task` function to see results.
@@ -542,7 +617,8 @@ class TheBlockchainAPIResource:
             "secret_recovery_phrase": secret_recovery_phrase,
             "network": network.value,
             "passphrase": passphrase,
-            "config_address": config_address
+            "config_address": config_address,
+            "candy_machine_contract_version": candy_machine_contract_version.value
         }
         if derivation_path is not None:
             payload['derivation_path'] = derivation_path
@@ -555,12 +631,72 @@ class TheBlockchainAPIResource:
             raise Exception(response['error_message'])
         return response['task_id']
 
+    def list_all_candy_machines(self):
+        """
+
+        :return:
+        """
+        response = self._request(
+            payload=dict(),
+            endpoint="solana/nft/candy_machine/list",
+            request_method=self.__RequestMethod.POST
+        )
+        if 'error_message' in response:
+            raise Exception(response['error_message'])
+        return response
+
+    def search_candy_machines(
+        self,
+        update_authority: Optional[str] = None,
+        update_authority_search_method: SearchMethod = SearchMethod.EXACT_MATCH,
+        config_address: Optional[str] = None,
+        config_address_search_method: SearchMethod = SearchMethod.EXACT_MATCH,
+        uuid: Optional[str] = None,
+        uuid_search_method: SearchMethod = SearchMethod.EXACT_MATCH,
+        symbol: Optional[str] = None,
+        symbol_search_method: SearchMethod = SearchMethod.EXACT_MATCH,
+        nft_name: Optional[str] = None,
+        nft_name_index: Optional[int] = None,
+        nft_name_search_method: SearchMethod = SearchMethod.EXACT_MATCH,
+        network: SolanaNetwork = SolanaNetwork.DEVNET,
+        candy_machine_contract_version: SolanaCandyMachineContractVersion = SolanaCandyMachineContractVersion.V1,
+    ):
+        payload = {
+            'network': network.value,
+            'candy_machine_contract_version': candy_machine_contract_version.value
+        }
+        if update_authority is not None:
+            payload['update_authority'] = update_authority
+            payload['update_authority_search_method'] = update_authority_search_method.value
+        if config_address is not None:
+            payload['config_address'] = config_address
+            payload['config_address_search_method'] = config_address_search_method.value
+        if uuid is not None:
+            payload['uuid'] = uuid
+            payload['uuid_search_method'] = uuid_search_method.value
+        if symbol is not None:
+            payload['symbol'] = symbol
+            payload['symbol_search_method'] = symbol_search_method.value
+        if nft_name is not None:
+            payload['nft_name'] = nft_name
+            payload['nft_name_index'] = nft_name_index
+            payload['nft_name_search_method'] = nft_name_search_method.value
+        response = self._request(
+            payload=payload,
+            endpoint="solana/nft/candy_machine/search",
+            request_method=self.__RequestMethod.POST
+        )
+        if 'error_message' in response:
+            raise Exception(response['error_message'])
+        return response
+
     def create_test_candy_machine(
         self,
         secret_recovery_phrase: str,
         derivation_path: Optional[str] = None,
         passphrase: str = str(),
-        network: SolanaNetwork = SolanaNetwork.DEVNET
+        network: SolanaNetwork = SolanaNetwork.DEVNET,
+        candy_machine_contract_version: SolanaCandyMachineContractVersion = SolanaCandyMachineContractVersion.V1,
     ):
         """
         Mint Info:
@@ -569,12 +705,14 @@ class TheBlockchainAPIResource:
         :param derivation_path:
         :param passphrase:
         :param network:
+        :param candy_machine_contract_version:
         :return:
         """
         payload = {
             "secret_recovery_phrase": secret_recovery_phrase,
             "network": network.value,
-            "passphrase": passphrase
+            "passphrase": passphrase,
+            "candy_machine_contract_version": candy_machine_contract_version.value
         }
         if derivation_path is not None:
             payload['derivation_path'] = derivation_path
@@ -587,22 +725,6 @@ class TheBlockchainAPIResource:
             raise Exception(response['error_message'])
         return response['candy_machine_address']
 
-    def get_task(self, task_id: str):
-        """
-        More Info:
-        https://docs.theblockchainapi.com/#operation/getTask
-        :param task_id:
-        :return:
-        """
-        response = self._request(
-            payload=dict(),
-            endpoint=f"task/{task_id}",
-            request_method=self.__RequestMethod.GET
-        )
-        if 'error_message' in response:
-            raise Exception(response['error_message'])
-        return response
-
     def get_solana_transaction(
         self,
         tx_signature: str,
@@ -623,49 +745,25 @@ class TheBlockchainAPIResource:
             raise Exception(response['error_message'])
         return response
 
-    def get_config_details(
-        self,
-        config_address,
-        network: SolanaNetwork = SolanaNetwork.DEVNET
-    ):
-        """
-        https://docs.theblockchainapi.com/#operation/solanaGetCandyMachineConfigurationDetails
-        :param config_address:
-        :param network:
-        :return:
-        """
-        payload = {
-            "network": network.value,
-            "config_address": config_address
-        }
-        response = self._request(
-            payload=payload,
-            endpoint="solana/nft/candy_machine/config/info",
-            request_method=self.__RequestMethod.POST
-        )
-        if 'error_message' in response:
-            raise Exception(response['error_message'])
-        return response
-
-    def get_nfts_minted_from_candy_machine(
+    def get_all_nfts_from_candy_machine(
         self,
         candy_machine_id,
         network: SolanaNetwork = SolanaNetwork.DEVNET
     ):
         """
-        https://docs.theblockchainapi.com/#operation/solanaGetNFTsMintedFromCandyMachine
+        Retrieves all NFTs from a Solana candy machine - both minted and unminted
+
+        See the returned attributes `all_nfts`, `unminted_nfts`, and `minted_nfts`
+
+        https://docs.theblockchainapi.com/#operation/solanaGetAllNFTsFromCandyMachine
         :param candy_machine_id:
         :param network:
         :return:
         """
-        payload = {
-            "network": network.value,
-            "candy_machine_id": candy_machine_id
-        }
         response = self._request(
-            payload=payload,
-            endpoint="solana/nft/candy_machine/nfts",
-            request_method=self.__RequestMethod.POST
+            payload={},
+            endpoint=f"solana/nft/candy_machine/{network.value}/{candy_machine_id}/nfts",
+            request_method=self.__RequestMethod.GET
         )
         if 'error_message' in response:
             raise Exception(response['error_message'])
